@@ -25,24 +25,25 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import logging
+import re
+import struct
+import sys
 from binascii import a2b_hex
+from typing import Optional
+
+import click
+
+from yubikit.core import TRANSPORT
 from yubikit.core.smartcard import (
+    AID,
+    SW,
+    ApduError,
     SmartCardConnection,
     SmartCardProtocol,
-    ApduFormat,
-    ApduError,
-    SW,
-    AID,
 )
-from .util import EnumChoice, CliFail, click_command
-from typing import Tuple, Optional
-
-import re
-import sys
-import click
-import struct
-import logging
 
+from .util import CliFail, EnumChoice, click_command
 
 logger = logging.getLogger(__name__)
 
@@ -51,6 +52,7 @@ APDU_PATTERN = re.compile(
     r"^"
     r"(?P<cla>[0-9a-f]{2})?(?P<ins>[0-9a-f]{2})(?P<params>[0-9a-f]{4})?"
     r"(?::(?P<body>(?:[0-9a-f]{2})+))?"
+    r"(?:/(?P<le>[0-9a-f]{2}))?"
     r"(?P<check>=(?P<sw>[0-9a-f]{4})?)?"
     r"$",
     re.IGNORECASE,
@@ -61,7 +63,9 @@ def _hex(data: bytes) -> str:
     return " ".join(f"{d:02X}" for d in data)
 
 
-def _parse_apdu(data: str) -> Tuple[Tuple[int, int, int, int, bytes], Optional[int]]:
+def _parse_apdu(
+    data: str,
+) -> tuple[tuple[int, int, int, int, bytes, int], Optional[int]]:
     m = APDU_PATTERN.match(data)
     if not m:
         raise ValueError("Invalid APDU format: " + data)
@@ -69,12 +73,13 @@ def _parse_apdu(data: str) -> Tuple[Tuple[int, int, int, int, bytes], Optional[i
     ins = int(m.group("ins"), 16)
     params = int(m.group("params") or "0000", 16)
     body = a2b_hex(m.group("body") or "")
+    le = int(m.group("le") or "00", 16)
     if m.group("check"):
         sw: Optional[int] = int(m.group("sw") or "9000", 16)
     else:
         sw = None
     p1, p2 = params >> 8, params & 0xFF
-    return (cla, ins, p1, p2, body), sw
+    return (cla, ins, p1, p2, body, le), sw
 
 
 def _print_response(resp: bytes, sw: int, no_pretty: bool) -> None:
@@ -103,17 +108,17 @@ def _print_response(resp: bytes, sw: int, no_pretty: bool) -> None:
     required=False,
     help="select application",
 )
-@click.option("--short", is_flag=True, help="use short APDUs instead of extended")
+@click.option("--short", is_flag=True, help="force usage of short APDUs")
 @click.argument("apdu", nargs=-1)
 @click.option("-s", "--send-apdu", multiple=True, help="provide full APDUs")
 def apdu(ctx, no_pretty, app, short, apdu, send_apdu):
     """
     Execute arbitrary APDUs.
     Provide APDUs as a hex encoded, space-separated list using the following syntax:
-    [CLA]INS[P1P2][:DATA][=EXPECTED_SW]
+    [CLA]INS[P1P2][:DATA][/LE][=EXPECTED_SW]
 
     If not provided CLA, P1 and P2 are all set to zero.
-    Setting EXPECTED_SW will cause the command to check the response SW an fail if it
+    Setting EXPECTED_SW will cause the command to check the response SW and fail if it
     differs. "=" can be used as shorthand for "=9000" (SW=OK).
 
     Examples:
@@ -132,6 +137,10 @@ def apdu(ctx, no_pretty, app, short, apdu, send_apdu):
       $ ykman apdu a40400:a000000527210101 04dead
         or (using full-apdu mode)
       $ ykman apdu -s 00a4040008a000000527210101 -s 0004dead
+
+    \b
+      Get 8 random bytes from the OpenPGP application:
+      $ ykman apdu -a openpgp 84/08=
     """
     if apdu and send_apdu:
         ctx.fail("Cannot mix positional APDUs and -s/--send-apdu.")
@@ -141,32 +150,10 @@ def apdu(ctx, no_pretty, app, short, apdu, send_apdu):
             ctx.fail("No commands provided.")
 
     dev = ctx.obj["device"]
-    scp_resolve = ctx.obj.get("scp")
 
     with dev.open_connection(SmartCardConnection) as conn:
-        protocol = SmartCardProtocol(conn)
         is_first = True
 
-        if scp_resolve:
-            params = scp_resolve(conn)
-        else:
-            params = None
-
-        if not short:
-            protocol.apdu_format = ApduFormat.EXTENDED
-        elif params:
-            ctx.fail("--short cannot be used with SCP")
-
-        if app:
-            is_first = False
-            click.echo("SELECT AID: " + _hex(app))
-            resp = protocol.select(app)
-            _print_response(resp, SW.OK, no_pretty)
-
-        if params:
-            click.echo("INITIALIZE SCP")
-            protocol.init_scp(params)
-
         if send_apdu:  # Compatibility mode (full APDUs)
             for apdu in send_apdu:
                 if not is_first:
@@ -175,18 +162,42 @@ def apdu(ctx, no_pretty, app, short, apdu, send_apdu):
                     is_first = False
                 apdu = a2b_hex(apdu)
                 click.echo("SEND: " + _hex(apdu))
-                resp, sw = protocol.connection.send_and_receive(apdu)
+                resp, sw = conn.send_and_receive(apdu)
                 _print_response(resp, sw, no_pretty)
         else:  # Standard mode
+            info = ctx.obj["info"]
+            protocol = SmartCardProtocol(conn)
+
+            scp_resolve = ctx.obj.get("scp")
+            if scp_resolve:
+                params = scp_resolve(conn)
+            else:
+                params = None
+
+            # Configure basic protocol settings
+            protocol.configure(info.version, force_short=short)
+
+            if app:
+                is_first = False
+                click.echo("SELECT AID: " + _hex(app))
+                resp = protocol.select(app)
+                _print_response(resp, SW.OK, no_pretty)
+
+            if params:
+                click.echo("INITIALIZE SCP")
+                protocol.init_scp(params)
+
             for apdu, check in apdus:
                 if not is_first:
                     click.echo()
                 else:
                     is_first = False
-                header, body = apdu[:4], apdu[4]
+                header, body, le = apdu[:4], apdu[4], apdu[5]
                 req = _hex(struct.pack(">BBBB", *header))
                 if body:
                     req += " -- " + _hex(body)
+                if le:
+                    req += f" (LE={le:02X})"
                 click.echo("SEND: " + req)
                 try:
                     resp = protocol.send_apdu(*apdu)
