@@ -25,42 +25,41 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import logging
+import os
+
+import click
+from cryptography.hazmat.primitives import serialization
+
 from yubikit.core import TRANSPORT
-from yubikit.core.smartcard import SmartCardConnection, ApduError, SW
+from yubikit.core.smartcard import SW, ApduError, SmartCardConnection
 from yubikit.hsmauth import (
+    ALGORITHM,
+    CREDENTIAL_PASSWORD_LEN,
+    MANAGEMENT_KEY_LEN,
     HsmAuthSession,
     InvalidPinError,
-    ALGORITHM,
-    MANAGEMENT_KEY_LEN,
-    CREDENTIAL_PASSWORD_LEN,
 )
 from yubikit.management import CAPABILITY
 
-from ..util import parse_private_key, InvalidPasswordError
-
 from ..hsmauth import (
-    get_hsmauth_info,
     generate_random_management_key,
+    get_hsmauth_info,
 )
+from ..util import InvalidPasswordError, parse_private_key
 from .util import (
     CliFail,
-    click_force_option,
-    click_postpone_execution,
     click_callback,
+    click_force_option,
     click_format_option,
-    click_prompt,
     click_group,
-    pretty_print,
+    click_postpone_execution,
+    click_prompt,
     get_scp_params,
     log_or_echo,
+    pretty_print,
 )
 
-from cryptography.hazmat.primitives import serialization
-
-import click
-import os
-import logging
-
 logger = logging.getLogger(__name__)
 
 
@@ -84,6 +83,8 @@ def handle_credential_error(
             raise CliFail("The device was not touched.")
         elif e.sw == SW.CONDITIONS_NOT_SATISFIED:
             raise CliFail(f"{target} does not meet complexity requirement.")
+    elif isinstance(e, ValueError):
+        raise  # Re-raise, ValueErrors are handled elsewhere
     raise CliFail(default_exception_msg)
 
 
@@ -681,8 +682,7 @@ def access():
     "-g",
     "--generate",
     is_flag=True,
-    help="generate a random management key "
-    "(can't be used with --new-management-key)",
+    help="generate a random management key (can't be used with --new-management-key)",
 )
 def change_management_key(ctx, management_key, new_management_key, generate):
     """
