@@ -25,29 +25,35 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
-from .core import (
-    int2bytes,
-    bytes2int,
-    require_version,
-    Version,
-    Tlv,
-    InvalidPinError,
-)
-from .core.smartcard import AID, SmartCardConnection, SmartCardProtocol, ApduError, SW
+import logging
+import struct
+from dataclasses import dataclass
+from enum import IntEnum, unique
+from functools import total_ordering
+from typing import NamedTuple, Optional, Union
 
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
-from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 from cryptography.hazmat.primitives.asymmetric import ec
+from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 
-
-from functools import total_ordering
-from enum import IntEnum, unique
-from dataclasses import dataclass
-from typing import Optional, List, Union, Tuple, NamedTuple
-import struct
-
-import logging
+from .core import (
+    InvalidPinError,
+    Tlv,
+    Version,
+    _override_version,
+    bytes2int,
+    int2bytes,
+    require_version,
+)
+from .core.smartcard import (
+    AID,
+    SW,
+    ApduError,
+    ScpKeyParams,
+    SmartCardConnection,
+    SmartCardProtocol,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -79,7 +85,7 @@ INS_PUT_MANAGEMENT_KEY = 0x08
 INS_GET_MANAGEMENT_KEY_RETRIES = 0x09
 INS_GET_PUBLIC_KEY = 0x0A
 
-# Lengths for paramters
+# Lengths for parameters
 MANAGEMENT_KEY_LEN = 16
 CREDENTIAL_PASSWORD_LEN = 16
 MIN_LABEL_LEN = 1
@@ -100,11 +106,12 @@ class ALGORITHM(IntEnum):
     EC_P256_YUBICO_AUTHENTICATION = 39
 
     @property
-    def key_len(self):
+    def key_len(self) -> int:
         if self.name.startswith("AES128"):
             return 16
         elif self.name.startswith("EC_P256"):
             return 32
+        raise ValueError("Unknown algorithm")
 
     @property
     def pubkey_len(self):
@@ -144,7 +151,7 @@ def _parse_select(response):
     return Version.from_bytes(data)
 
 
-def _password_to_key(password: str) -> Tuple[bytes, bytes]:
+def _password_to_key(password: str) -> tuple[bytes, bytes]:
     """Derive encryption and MAC key from a password.
 
     :return: A tuple containing the encryption key, and MAC key.
@@ -213,9 +220,19 @@ class SessionKeys(NamedTuple):
 class HsmAuthSession:
     """A session with the YubiHSM Auth application."""
 
-    def __init__(self, connection: SmartCardConnection) -> None:
+    def __init__(
+        self,
+        connection: SmartCardConnection,
+        scp_key_params: Optional[ScpKeyParams] = None,
+    ) -> None:
         self.protocol = SmartCardProtocol(connection)
-        self._version = _parse_select(self.protocol.select(AID.HSMAUTH))
+        self._version = _override_version.patch(
+            _parse_select(self.protocol.select(AID.HSMAUTH))
+        )
+
+        self.protocol.configure(self._version)
+        if scp_key_params:
+            self.protocol.init_scp(scp_key_params)
 
     @property
     def version(self) -> Version:
@@ -227,7 +244,7 @@ class HsmAuthSession:
         self.protocol.send_apdu(0, INS_RESET, 0xDE, 0xAD)
         logger.info("YubiHSM Auth application data reset performed")
 
-    def list_credentials(self) -> List[Credential]:
+    def list_credentials(self) -> list[Credential]:
         """List YubiHSM Auth credentials on YubiKey"""
 
         creds = []
@@ -411,7 +428,7 @@ class HsmAuthSession:
         return self._put_credential(
             management_key,
             label,
-            b"",  # Emtpy byte will generate key
+            b"",  # Empty byte will generate key
             ALGORITHM.EC_P256_YUBICO_AUTHENTICATION,
             credential_password,
             touch_required,
@@ -599,14 +616,27 @@ class HsmAuthSession:
             )
         )
 
-    def get_challenge(self, label: str) -> bytes:
+    def get_challenge(
+        self, label: str, credential_password: Union[bytes, str, None] = None
+    ) -> bytes:
         """Get the Host Challenge.
 
-        For symmetric credentials this is Host Challenge, a random
-        8 byte value. For asymmetric credentials this is EPK-OCE.
+        For symmetric credentials this is Host Challenge, a random 8 byte value.
+        For asymmetric credentials this is EPK-OCE.
 
         :param label: The label of the credential.
+        :param credential_password: The password used to protect access to the
+            credential, needed for asymmetric credentials.
         """
         require_version(self.version, (5, 6, 0))
-        data = Tlv(TAG_LABEL, _parse_label(label))
+
+        data: bytes = Tlv(TAG_LABEL, _parse_label(label))
+
+        if credential_password is not None and (
+            self.version >= (5, 7, 1) or self.version[0] == 0
+        ):
+            data += Tlv(
+                TAG_CREDENTIAL_PASSWORD, _parse_credential_password(credential_password)
+            )
+
         return self.protocol.send_apdu(0, INS_GET_CHALLENGE, 0, 0, data)
