@@ -25,38 +25,39 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+import logging
+from dataclasses import replace
+from typing import Optional
+
 from .core import (
+    PID,
     TRANSPORT,
     YUBIKEY,
-    PID,
-    Version,
+    ApplicationNotAvailableError,
     Connection,
     NotSupportedError,
-    ApplicationNotAvailableError,
+    Version,
 )
-from .core.otp import OtpConnection, CommandRejectedError
 from .core.fido import FidoConnection
+from .core.otp import OtpConnection
 from .core.smartcard import (
     AID,
     SmartCardConnection,
     SmartCardProtocol,
 )
 from .management import (
-    ManagementSession,
-    DeviceInfo,
-    DeviceConfig,
-    Mode,
-    USB_INTERFACE,
     CAPABILITY,
-    FORM_FACTOR,
     DEVICE_FLAG,
+    FORM_FACTOR,
+    USB_INTERFACE,
+    DeviceConfig,
+    DeviceInfo,
+    ManagementSession,
+    Mode,
+    VersionQualifier,
 )
 from .yubiotp import YubiOtpSession
 
-from time import sleep
-from typing import Optional
-import logging
-
 logger = logging.getLogger(__name__)
 
 
@@ -83,8 +84,10 @@ def _read_info_ccid(conn, key_type, interfaces):
         try:
             return mgmt.read_device_info()
         except NotSupportedError:
-            # Workaround to "de-select" the Management Applet needed for NEO
-            conn.send_and_receive(b"\xa4\x04\x00\x08")
+            if version.major == 3:
+                # Workaround to "de-select" the Management Applet needed for NEO
+                logger.debug("Send NEO de-select workaround...")
+                conn.send_and_receive(b"\xa4\x04\x00\x08")
     except ApplicationNotAvailableError:
         logger.debug("Couldn't select Management application, use fallback")
 
@@ -148,40 +151,22 @@ def _read_info_ccid(conn, key_type, interfaces):
             TRANSPORT.NFC: capabilities,
         },
         is_locked=False,
+        version_qualifier=VersionQualifier(version),
     )
 
 
 def _read_info_otp(conn, key_type, interfaces):
-    otp = None
-    serial = None
-
     try:
         mgmt = ManagementSession(conn)
-    except ApplicationNotAvailableError:
-        otp = YubiOtpSession(conn)
-
-    # Retry during potential reclaim timeout period (~3s).
-    for _ in range(8):
-        try:
-            if otp is None:
-                try:
-                    return mgmt.read_device_info()  # Rejected while reclaim
-                except NotSupportedError:
-                    otp = YubiOtpSession(conn)
-            serial = otp.get_serial()  # Rejected if reclaim (or not API_SERIAL_VISIBLE)
-            break
-        except CommandRejectedError:
-            if otp and interfaces == USB_INTERFACE.OTP:
-                break  # Can't be reclaim with only one interface
-            logger.debug("Potential reclaim, sleep...", exc_info=True)
-            sleep(0.5)  # Potential reclaim
-    else:
-        otp = YubiOtpSession(conn)
+        return mgmt.read_device_info()
+    except (ApplicationNotAvailableError, NotSupportedError):
+        logger.debug("Unable to get info via Management application, use fallback")
 
     # Synthesize info
-    logger.debug("Unable to get info via Management application, use fallback")
-
+    otp = YubiOtpSession(conn)
+    serial = otp.get_serial()
     version = otp.version
+
     if key_type == YUBIKEY.NEO:
         usb_supported = _BASE_NEO_APPS
         if USB_INTERFACE.FIDO in interfaces or version >= (3, 3, 0):
@@ -211,6 +196,7 @@ def _read_info_otp(conn, key_type, interfaces):
         form_factor=FORM_FACTOR.UNKNOWN,
         supported_capabilities=capabilities.copy(),
         is_locked=False,
+        version_qualifier=VersionQualifier(version),
     )
 
 
@@ -244,6 +230,7 @@ def _read_info_ctap(conn, key_type, interfaces):
             form_factor=FORM_FACTOR.USB_A_KEYCHAIN,
             supported_capabilities=supported_apps,
             is_locked=False,
+            version_qualifier=VersionQualifier(version),
         )
 
 
@@ -326,17 +313,14 @@ def read_info(conn: Connection, pid: Optional[PID] = None) -> DeviceInfo:
     if (4, 4, 0) <= info.version < (4, 5, 0):
         info.is_fips = True
 
-    # Set nfc_enabled if missing (pre YubiKey 5)
-    if (
-        info.has_transport(TRANSPORT.NFC)
-        and TRANSPORT.NFC not in info.config.enabled_capabilities
-    ):
-        info.config.enabled_capabilities[TRANSPORT.NFC] = info.supported_capabilities[
-            TRANSPORT.NFC
-        ]
-
-    # Workaround for invalid configurations.
-    if info.version >= (4, 0, 0):
+    # Fix NFC if needed
+    if info.has_transport(TRANSPORT.NFC):
+        # Set nfc_enabled if missing (pre YubiKey 5)
+        if TRANSPORT.NFC not in info.config.enabled_capabilities:
+            info.config.enabled_capabilities[TRANSPORT.NFC] = (
+                info.supported_capabilities[TRANSPORT.NFC]
+            )
+        # Workaround for invalid configurations
         if info.form_factor in (
             FORM_FACTOR.USB_A_NANO,
             FORM_FACTOR.USB_C_NANO,
@@ -344,9 +328,11 @@ def read_info(conn: Connection, pid: Optional[PID] = None) -> DeviceInfo:
         ) or (
             info.form_factor is FORM_FACTOR.USB_C_KEYCHAIN and info.version < (5, 2, 4)
         ):
-            # Known not to have NFC
-            info.supported_capabilities.pop(TRANSPORT.NFC, None)
-            info.config.enabled_capabilities.pop(TRANSPORT.NFC, None)
+            # Known to not have NFC, remove capabilities
+            supported = dict(info.supported_capabilities)
+            del supported[TRANSPORT.NFC]
+            replace(info, supported_capabilities=supported)
+            del info.config.enabled_capabilities[TRANSPORT.NFC]
 
     logger.debug("Device info, after tweaks: %s", info)
     return info
